@@ -77,6 +77,10 @@ extern int errno;
 int lstat PARAMS((const char *__path, struct stat *__statbuf));
 int stat PARAMS((const char *__path, struct stat *__statbuf));
 
+int optionl_stat PARAMS((const char *name, struct stat *p));
+int optionp_stat PARAMS((const char *name, struct stat *p));
+int optionh_stat PARAMS((const char *name, struct stat *p));
+
 #ifndef S_ISUID
 # define S_ISUID 0004000
 #endif
@@ -322,6 +326,20 @@ char *xstrdup PARAMS((char *string));
 
 /* find global function declarations.  */
 
+/* find.c */
+/* SymlinkOption represents the choice of 
+ * -P, -L or -P (default) on the command line.
+ */
+enum SymlinkOption 
+  {
+    SYMLINK_ALWAYS_DEREF,	/* Option -L */
+    SYMLINK_NEVER_DEREF,	/* Option -P */
+    SYMLINK_DEREF_ARGSONLY	/* Option -H */
+  };
+extern enum SymlinkOption symlink_handling; /* defined in find.c. */
+
+void set_follow_state PARAMS((enum SymlinkOption opt));
+
 /* fstype.c */
 char *filesystem_type PARAMS((const char *path, const char *relpath, const struct stat *statp));
 
