@@ -25,6 +25,9 @@
 
 #include "defs.h"
 
+#include <errno.h>
+
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #else
@@ -56,6 +59,7 @@
 #define apply_predicate(pathname, stat_buf_ptr, node)	\
   (*(node)->pred_func)((pathname), (stat_buf_ptr), (node))
 
+
 static void process_top_path PARAMS((char *pathname));
 static int process_path PARAMS((char *pathname, char *name, boolean leaf, char *parent));
 static void process_dir PARAMS((char *pathname, char *name, int pathlen, struct stat *statp, char *parent));
@@ -135,7 +139,9 @@ char *rel_pathname;
 int path_length;
 
 /* true if following symlinks.  Should be consistent with xstat.  */
-boolean dereference;
+/* boolean dereference; */
+enum SymlinkOption symlink_handling;
+
 
 /* Pointer to the function used to stat files. */
 int (*xstat) ();
@@ -154,16 +160,118 @@ boolean ignore_readdir_race;
 boolean warnings;
 
 
+
+/* optionh_stat() implements the stat operation when the -H option is
+ * in effect.
+ * 
+ * If the item to be examined is a command-line argument, we follow
+ * symbolic links.  If the stat() call fails on the command-line item,
+ * we fall back on the properties of the symbolic link.
+ *
+ * If the item to be examined is not a command-line argument, we
+ * examine the link itself.
+ */
+int 
+optionh_stat(const char *name, struct stat *p)
+{
+  if (0 == curdepth) 
+    {
+      /* This file is from the command line; deference the link (if it
+       * is a link).  
+       */
+      if (0 == stat(name, p))
+	{
+	  /* success */
+	  return 0;
+	}
+      else
+	{
+	  /* fallback - return the information for the link itself. */
+	  return lstat(name, p);
+	}
+    }
+  else
+    {
+      /* Not a file on the command line; do not derefernce the link.
+       */
+      return lstat(name, p);
+    }
+}
+
+/* optionl_stat() implements the stat operation when the -L option is
+ * in effect.  That option makes us examine the thing the symbolic
+ * link points to, not the symbolic link itself.
+ */
+int 
+optionl_stat(const char *name, struct stat *p)
+{
+  if (0 == stat(name, p))
+    {
+      return 0;			/* normal case. */
+    }
+  else
+    {
+      return lstat(name, p);	/* can't follow link, return the link itself. */
+    }
+}
+
+/* optionp_stat() implements the stat operation when the -P option is
+ * in effect (this is also the default).  That option makes us examine
+ * the symbolic link itself, not the thing it points to.
+ */
+int 
+optionp_stat(const char *name, struct stat *p)
+{
+  return lstat(name, p);
+}
+
 #ifdef DEBUG_STAT
 static int
-debug_stat (file, bufp)
-     char *file;
-     struct stat *bufp;
+debug_stat (const char *file, struct stat *bufp)
 {
   fprintf (stderr, "debug_stat (%s)\n", file);
-  return lstat (file, bufp);
+  switch (symlink_handling)
+    {
+    case SYMLINK_ALWAYS_DEREF:
+      return optionl_stat(file, bufp);
+    case SYMLINK_DEREF_ARGSONLY:
+      return optionh_stat(file, bufp);
+    case SYMLINK_NEVER_DEREF:
+      return optionp_stat(file, bufp);
+    }
 }
 #endif /* DEBUG_STAT */
+
+void 
+set_follow_state(enum SymlinkOption opt)
+{
+  switch (opt)
+    {
+    case SYMLINK_ALWAYS_DEREF:  /* -L */
+      xstat = optionl_stat;
+      no_leaf_check = false;
+      break;
+      
+    case SYMLINK_NEVER_DEREF:	/* -P (default) */
+      xstat = optionp_stat;
+      /* Can't turn on no_leaf_check because the user might have specified 
+       * -noleaf anyway
+       */
+      break;
+      
+    case SYMLINK_DEREF_ARGSONLY: /* -H */
+      xstat = optionh_stat;
+      no_leaf_check = true;
+    }
+  
+  /* For DBEUG_STAT, the choice is made at runtime within debug_stat()
+   * by checking the contents of the symlink_handling variable.
+   */
+#if defined(DEBUG_STAT)
+  xstat = debug_stat;
+#endif /* !DEBUG_STAT */
+}
+
 
 int
 main (int argc, char **argv)
@@ -172,7 +280,7 @@ main (int argc, char **argv)
   PFB parse_function;		/* Pointer to the function which parses. */
   struct predicate *cur_pred;
   char *predicate_name;		/* Name of predicate being parsed. */
-
+  int end_of_leading_options = 0; /* First arg after any -H/-L etc. */
   program_name = argv[0];
 
 #ifdef HAVE_SETLOCALE
@@ -202,11 +310,9 @@ main (int argc, char **argv)
   stay_on_filesystem = false;
   ignore_readdir_race = false;
   exit_status = 0;
-  dereference = false;
-#ifdef DEBUG_STAT
+
+#if defined(DEBUG_STAT)
   xstat = debug_stat;
-#else /* !DEBUG_STAT */
-  xstat = lstat;
 #endif /* !DEBUG_STAT */
 
 #if 0  
@@ -221,6 +327,8 @@ main (int argc, char **argv)
     {
       error (1, 0, _("The environment variable FIND_BLOCK_SIZE is not supported, the only thing that affects the block size is the POSIXLY_CORRECT environment variable"));
     }
+
+  set_follow_state(SYMLINK_NEVER_DEREF); /* The default is equivalent to -P. */
   
 #endif
 
@@ -228,10 +336,54 @@ main (int argc, char **argv)
   printf ("cur_day_start = %s", ctime (&cur_day_start));
 #endif /* DEBUG */
 
-  /* Find where in ARGV the predicates begin. */
-  for (i = 1; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
-    /* Do nothing. */ ;
+  /* Check for -P, -H or -L options. */
+  for (i=1; (end_of_leading_options = i) < argc; ++i)
+    {
+      if (0 == strcmp("-H", argv[i]))
+	{
+	  /* Meaning: dereference symbolic links on command line, but nowhere else. */
+	  set_follow_state(SYMLINK_DEREF_ARGSONLY);
+	}
+      else if (0 == strcmp("-L", argv[i]))
+	{
+	  /* Meaning: dereference all symbolic links. */
+	  set_follow_state(SYMLINK_ALWAYS_DEREF);
+	}
+      else if (0 == strcmp("-P", argv[i]))
+	{
+	  /* Meaning: never dereference symbolic links (default). */
+	  set_follow_state(SYMLINK_NEVER_DEREF);
+	}
+      else if (0 == strcmp("--", argv[i]))
+	{
+	  /* -- signifies the end of options. */
+	  end_of_leading_options = i+1;	/* Next time start with the next option */
+	  break;
+	}
+      else
+	{
+	  /* Hmm, must be one of 
+	   * (a) A path name
+	   * (b) A predicate
+	   */
+	  end_of_leading_options = i; /* Next time start with this option */
+	  break;
+	}
+    }
 
+  /* We are now processing the part of the "find" command line 
+   * after the -H/-L options (if any).
+   */
+
+  /* fprintf(stderr, "rest: optind=%ld\n", (long)optind); */
+  
+  /* Find where in ARGV the predicates begin. */
+  for (i = end_of_leading_options; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
+    {
+      /* fprintf(stderr, "Looks like %s is not a predicate\n", argv[i]); */
+      /* Do nothing. */ ;
+    }
+  
   /* Enclose the expression in `( ... )' so a default -print will
      apply to the whole expression. */
   parse_open (argv, &argc);
@@ -329,9 +481,13 @@ main (int argc, char **argv)
     error (1, errno, _("cannot get current directory"));
 
   /* If no paths are given, default to ".".  */
-  for (i = 1; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
-    process_top_path (argv[i]);
-  if (i == 1)
+  for (i = end_of_leading_options; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
+    {
+      process_top_path (argv[i]);
+    }
+
+  /* If there were no path arguments, default to ".". */
+  if (i == end_of_leading_options)
     {
       /* 
        * We use a temporary variable here because some actions modify 
@@ -702,8 +858,15 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 	  /* We could go back and do the next command-line arg
 	     instead, maybe using longjmp.  */
 	  char const *dir;
-
-	  if (!dereference)
+	  boolean deref;
+	  if (symlink_handling == SYMLINK_ALWAYS_DEREF)
+	    deref = true;
+	  else if (symlink_handling == SYMLINK_DEREF_ARGSONLY && (curdepth == 0))
+	    deref = true;
+	  else
+	    deref = false;
+	  
+	  if (!deref)
 	    dir = "..";
 	  else
 	    {
@@ -731,10 +894,10 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 		  tmp.st_ino = starting_stat_buf.st_ino;
 		}
 
-	      problem_is_with_parent = dereference ? 1 : 0;
+	      problem_is_with_parent = deref ? 1 : 0;
 	      wd_sanity_check(pathname,
 			      program_name,
-			      dereference ? parent : starting_dir,
+			      deref ? parent : starting_dir,
 			      &tmp, &stat_buf,
 			      problem_is_with_parent, __LINE__);
 	    }
