@@ -251,7 +251,7 @@ static struct parser_table const parse_table[] =
   {ARG_TEST,               "wholename",             parse_wholename},   /* GNU, replaces -path */
   {ARG_TEST,               "xdev",                  parse_xdev},
   {ARG_TEST,               "xtype",                 parse_xtype},	/* GNU */
-  {0, 0}
+  {0, 0, 0}
 };
 
 
@@ -509,7 +509,9 @@ parse_delete (argv, arg_ptr)
   int *arg_ptr;
 {
   struct predicate *our_pred;
-
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = insert_primary (pred_delete);
   our_pred->side_effects = true;
   /* -delete implies -depth */
@@ -609,10 +611,8 @@ parse_follow (char **argv, int *arg_ptr)
   (void) argv;
   (void) arg_ptr;
 
-  dereference = true;
-  xstat = stat;
-  no_leaf_check = true;
-  return (true);
+  set_follow_state(SYMLINK_ALWAYS_DEREF);
+  return true;
 }
 
 static boolean
@@ -1082,6 +1082,9 @@ parse_nouser (char **argv, int *arg_ptr)
 static boolean
 parse_nowarn (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
+  
   warnings = false;
   return true;;
 }
@@ -1212,6 +1215,9 @@ parse_print (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = insert_primary (pred_print);
   /* -print has the side effect of printing.  This prevents us
      from doing undesired multiple printing when the user has
@@ -1227,6 +1233,9 @@ parse_print0 (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = insert_primary (pred_print0);
   /* -print0 has the side effect of printing.  This prevents us
      from doing undesired multiple printing when the user has
@@ -1250,6 +1259,9 @@ parse_prune (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = insert_primary (pred_prune);
   our_pred->need_stat = false;
   /* -prune has a side effect that it does not descend into
@@ -1262,7 +1274,9 @@ static boolean
 parse_quit  (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred = insert_primary (pred_quit);
-  (*arg_ptr)++;
+  (void) argv;
+  (void) arg_ptr;
+  our_pred->need_stat = false;
   return true;
 }
 
@@ -1385,6 +1399,9 @@ parse_true (char **argv, int *arg_ptr)
 {
   struct predicate *our_pred;
 
+  (void) argv;
+  (void) arg_ptr;
+  
   our_pred = insert_primary (pred_true);
   our_pred->need_stat = false;
   return (true);
@@ -1454,7 +1471,9 @@ static boolean
 parse_version (char **argv, int *arg_ptr)
 {
   extern char *version_string;
-
+  (void) argv;
+  (void) arg_ptr;
+  
   fflush (stderr);
   printf (_("GNU find version %s\n"), version_string);
   exit (0);
@@ -1463,6 +1482,8 @@ parse_version (char **argv, int *arg_ptr)
 static boolean
 parse_xdev (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
   stay_on_filesystem = true;
   return true;
 }
@@ -1470,6 +1491,8 @@ parse_xdev (char **argv, int *arg_ptr)
 static boolean
 parse_ignore_race (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
   ignore_readdir_race = true;
   return true;
 }
@@ -1477,6 +1500,8 @@ parse_ignore_race (char **argv, int *arg_ptr)
 static boolean
 parse_noignore_race (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
   ignore_readdir_race = false;
   return true;
 }
@@ -1484,6 +1509,8 @@ parse_noignore_race (char **argv, int *arg_ptr)
 static boolean
 parse_warn (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
   warnings = true;
   return true;
 }
@@ -1491,6 +1518,8 @@ parse_warn (char **argv, int *arg_ptr)
 static boolean
 parse_xtype (char **argv, int *arg_ptr)
 {
+  (void) argv;
+  (void) arg_ptr;
   return insert_type (argv, arg_ptr, pred_xtype);
 }
 
@@ -1646,7 +1675,7 @@ insert_fprintf (FILE *fp, boolean (*func) (/* ??? */), char **argv, int *arg_ptr
 	  if (*scan2 == '.')
 	    for (scan2++; ISDIGIT (*scan2); scan2++)
 	      /* Do nothing. */ ;
-	  if (strchr ("abcdDfFgGhHiklmnpPstuUyY", *scan2))
+	  if (strchr ("abcdDfFgGhHiklmMnpPstuUyY", *scan2))
 	    {
 	      segmentp = make_segment (segmentp, format, scan2 - format,
 				       (int) *scan2);
@@ -1707,21 +1736,16 @@ make_segment (struct segment **segment, char *format, int len, int kind)
 
     case 'a':			/* atime in `ctime' format */
     case 'A':			/* atime in user-specified strftime format */
-    case 'b':			/* size in 512-byte blocks */
     case 'c':			/* ctime in `ctime' format */
     case 'C':			/* ctime in user-specified strftime format */
-    case 'D':                   /* Filesystem device on which the file exits */
     case 'F':			/* filesystem type */
-    case 'G':			/* GID number */
     case 'g':			/* group name */
     case 'i':			/* inode number */
-    case 'k':			/* size in 1K blocks */
     case 'l':			/* object of symlink */
-    case 'n':			/* number of links */
+    case 'M':			/* mode in `ls -l' format (eg., "drwxr-xr-x") */
     case 's':			/* size in bytes */
     case 't':			/* mtime in `ctime' format */
     case 'T':			/* mtime in user-specified strftime format */
-    case 'U':			/* UID number */
     case 'u':			/* user name */
     case 'y':			/* file type */
     case 'Y':			/* symlink pointed file type */
@@ -1735,6 +1759,20 @@ make_segment (struct segment **segment, char *format, int len, int kind)
       *fmt++ = 's';
       break;
 
+      /* Numeric items that one might expect to honour 
+       * #, 0, + flags but which do not.
+       */
+    case 'G':			/* GID number */
+    case 'U':			/* UID number */
+    case 'b':			/* size in 512-byte blocks */
+    case 'D':                   /* Filesystem device on which the file exits */
+    case 'k':			/* size in 1K blocks */
+    case 'n':			/* number of links */
+      fprintf_stat_needed = true;
+      *fmt++ = 's';
+      
+      /* Numeric items that DO honour #, 0, + flags.
+       */
     case 'd':			/* depth in search tree (0 = ARGV element) */
       *fmt++ = 'd';
       break;
