@@ -26,6 +26,7 @@
 #include "../gnulib/lib/dirname.h"
 #include "../gnulib/lib/human.h"
 #include "modetype.h"
+#include "filemode.h"
 #include "wait.h"
 
 #if ENABLE_NLS
@@ -332,7 +333,7 @@ pred_close (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
   (void) &stat_buf;
   (void) &pred_ptr;
   
-  return (true);
+  return true;
 }
 
 boolean
@@ -345,11 +346,12 @@ pred_cmin (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_cnewer (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  (void) &pathname;
+  (void) pathname;
   
   if (stat_buf->st_ctime > pred_ptr->args.time)
-    return (true);
-  return (false);
+    return true;
+  else
+    return false;
 }
 
 boolean
@@ -381,11 +383,10 @@ pred_ctime (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 }
 
 boolean
-pred_delete (pathname, stat_buf, pred_ptr)
-     char *pathname;
-     struct stat *stat_buf;
-     struct predicate *pred_ptr;
+pred_delete (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pred_ptr;
+  (void) stat_buf;
   if (strcmp (rel_pathname, "."))
     {
       if (0 != remove (rel_pathname))
@@ -406,8 +407,8 @@ pred_delete (pathname, stat_buf, pred_ptr)
 boolean
 pred_empty (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  (void) &pathname;
-  (void) &pred_ptr;
+  (void) pathname;
+  (void) pred_ptr;
   
   if (S_ISDIR (stat_buf->st_mode))
     {
@@ -450,13 +451,13 @@ pred_empty (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_exec (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  (void) &pathname;
-  (void) &stat_buf;
-  
   int i;
   int path_pos;
   struct exec_val *execp;	/* Pointer for efficiency. */
 
+  (void) pathname;
+  (void) stat_buf;
+  
   execp = &pred_ptr->args.exec_vec;
 
   /* Replace "{}" with the real path in each affected arg. */
@@ -672,6 +673,16 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	  }
 #endif				/* S_ISLNK */
 	  break;
+	  
+	case 'M':		/* mode as 10 chars (eg., "-rwxr-x--x" */
+	  {
+	    char modestring[16] ;
+	    mode_string (stat_buf->st_mode, modestring);
+	    modestring[10] = '\0';
+	    fprintf (fp, segment->text, modestring);
+	  }
+	  break;
+	  
 	case 'm':		/* mode as octal number (perms only) */
 	  {
 	    /* Output the mode portably using the traditional numbers,
@@ -701,6 +712,7 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 			 | (m & S_IXOTH ? 00001 : 0))));
 	  }
 	  break;
+	  
 	case 'n':		/* number of links */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) stat_buf->st_nlink,
@@ -747,6 +759,7 @@ pred_fprintf (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 	      }
 	    /* else fallthru */
 	  }
+	  
 	case 'U':		/* UID number */
 	  fprintf (fp, segment->text,
 		   human_readable ((uintmax_t) stat_buf->st_uid, hbuf,
@@ -856,10 +869,10 @@ pred_ilname (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_iname (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  (void) stat_buf;
-
   const char *base;
 
+  (void) stat_buf;
+
   /* FNM_PERIOD is not used here because POSIX requires that it not be.
    * See http://standards.ieee.org/reading/ieee/interp/1003-2-92_int/pasc-1003.2-126.html
    */
@@ -905,6 +918,8 @@ pred_ipath (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_links (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
+  
   switch (pred_ptr->args.info.kind)
     {
     case COMP_GT:
@@ -952,6 +967,8 @@ insert_lname (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr,
 boolean
 pred_ls (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pred_ptr;
+  
   list_file (pathname, rel_pathname, stat_buf, start_time,
 	     output_block_size, stdout);
   return (true);
@@ -976,6 +993,7 @@ pred_name (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   const char *base;
 
+  (void) stat_buf;
   base = base_name (pathname);
 
   /* FNM_PERIOD is not used here because POSIX requires that it not be.
@@ -1007,6 +1025,8 @@ pred_negate (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_newer (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
+  
   if (stat_buf->st_mtime > pred_ptr->args.time)
     return (true);
   return (false);
@@ -1015,6 +1035,9 @@ pred_newer (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_nogroup (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
+  (void) pred_ptr;
+  
 #ifdef CACHE_IDS
   extern char *gid_unused;
 
@@ -1029,7 +1052,12 @@ pred_nouser (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
 #ifdef CACHE_IDS
   extern char *uid_unused;
-
+#endif
+  
+  (void) pathname;
+  (void) pred_ptr;
+  
+#ifdef CACHE_IDS
   return uid_unused[(unsigned) stat_buf->st_uid];
 #else
   return getpwuid (stat_buf->st_uid) == NULL;
@@ -1057,7 +1085,10 @@ pred_ok (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_open (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  return (true);
+  (void) pathname;
+  (void) stat_buf;
+  (void) pred_ptr;
+  return true;
 }
 
 boolean
@@ -1088,6 +1119,7 @@ pred_or (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_path (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) stat_buf;
   if (fnmatch (pred_ptr->args.str, pathname, 0) == 0)
     return (true);
   return (false);
@@ -1096,6 +1128,7 @@ pred_path (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_perm (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
   switch (pred_ptr->args.perm.kind)
     {
     case PERM_AT_LEAST:
@@ -1119,13 +1152,17 @@ pred_perm (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_print (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) stat_buf;
+  (void) pred_ptr;
   puts (pathname);
-  return (true);
+  return true;
 }
 
 boolean
 pred_print0 (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) stat_buf;
+  (void) pred_ptr;
   fputs (pathname, stdout);
   putc (0, stdout);
   return (true);
@@ -1134,6 +1171,9 @@ pred_print0 (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_prune (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
+  (void) stat_buf;
+  (void) pred_ptr;
   stop_at_current_level = true;
   return (do_dir_first);	/* This is what SunOS find seems to do. */
 }
@@ -1141,9 +1181,9 @@ pred_prune (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_quit (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  (void) &pathname;
-  (void) &stat_buf;
-  (void) &pred_ptr;
+  (void) pathname;
+  (void) stat_buf;
+  (void) pred_ptr;
   exit(0);
 }
 
@@ -1151,6 +1191,7 @@ boolean
 pred_regex (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   int len = strlen (pathname);
+(void) stat_buf;
   if (re_match (pred_ptr->args.regex, pathname, len, 0,
 		(struct re_registers *) NULL) == len)
     return (true);
@@ -1162,6 +1203,7 @@ pred_size (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   uintmax_t f_val;
 
+  (void) pathname;
   f_val = ((stat_buf->st_size / pred_ptr->args.size.blocksize)
 	   + (stat_buf->st_size % pred_ptr->args.size.blocksize != 0));
   switch (pred_ptr->args.size.kind)
@@ -1185,7 +1227,10 @@ pred_size (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_true (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  return (true);
+  (void) pathname;
+  (void) stat_buf;
+  (void) pred_ptr;
+  return true;
 }
 
 boolean
@@ -1194,6 +1239,8 @@ pred_type (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
   mode_t mode = stat_buf->st_mode;
   mode_t type = pred_ptr->args.type;
 
+  (void) pathname;
+  
 #ifndef S_IFMT
   /* POSIX system; check `mode' the slow way. */
   if ((S_ISBLK (mode) && type == S_IFBLK)
@@ -1225,6 +1272,7 @@ pred_type (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_uid (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
   switch (pred_ptr->args.info.kind)
     {
     case COMP_GT:
@@ -1248,6 +1296,7 @@ pred_used (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
   time_t delta;
 
+  (void) pathname;
   delta = stat_buf->st_atime - stat_buf->st_ctime; /* Use difftime? */
   return pred_timewindow(delta, pred_ptr, DAYSECS);
 }
@@ -1255,6 +1304,7 @@ pred_used (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_user (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
+  (void) pathname;
   if (pred_ptr->args.uid == stat_buf->st_uid)
     return (true);
   else
@@ -1264,19 +1314,38 @@ pred_user (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 boolean
 pred_xtype (char *pathname, struct stat *stat_buf, struct predicate *pred_ptr)
 {
-  struct stat sbuf;
-  int (*ystat) ();
+  struct stat sbuf;		/* local copy, not stat_buf because we're using a different stat method */
+  int (*ystat) (const char*, struct stat *p);
 
-  ystat = xstat == lstat ? stat : lstat;
+  switch (symlink_handling)
+    {
+    case SYMLINK_ALWAYS_DEREF:
+      ystat = optionl_stat;
+    case SYMLINK_DEREF_ARGSONLY:
+    case SYMLINK_NEVER_DEREF:
+      ystat = optionp_stat;
+    }
+  
   if ((*ystat) (rel_pathname, &sbuf) != 0)
     {
-      if (ystat == stat && errno == ENOENT)
-	/* Mimic behavior of ls -lL. */
-	return (pred_type (pathname, stat_buf, pred_ptr));
-      error (0, errno, "%s", pathname);
-      exit_status = 1;
-      return (false);
+      if (ystat == optionl_stat && errno == ENOENT)
+	{
+	  /* If we failed to follow the symlink,
+	   * fall back on looking at the symlink itself. 
+	   */
+	  /* Mimic behavior of ls -lL. */
+	  return (pred_type (pathname, stat_buf, pred_ptr));
+	}
+      else
+	{
+	  error (0, errno, "%s", pathname);
+	  exit_status = 1;
+	}
+      return false;
     }
+  /* Now that we have our stat() information, query it in the same 
+   * way that -type does.
+   */
   return (pred_type (pathname, &sbuf, pred_ptr));
 }
 
@@ -1363,11 +1432,13 @@ format_date (time_t when, int kind)
 {
   static char buf[MAX (LONGEST_HUMAN_READABLE + 2, 64)];
   struct tm *tm;
-  char fmt[3];
+  char fmt[6];
 
   fmt[0] = '%';
   fmt[1] = kind;
   fmt[2] = '\0';
+  if (kind == '+')
+    strcpy (fmt, "%F+%T");
 
   if (kind != '@'
       && (tm = localtime (&when))
