@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 9 Temple Place - Suite 330, Boston, MA 02111-1307,
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
    USA.
 */
 
@@ -45,13 +45,6 @@
    Written by James A. Woods <jwoods@adobe.com>.
    Modified by David MacKenzie <djm@gnu.ai.mit.edu>.  */
 
-#include <gnulib/config.h>
-#undef VERSION
-#undef PACKAGE_VERSION
-#undef PACKAGE_TARNAME
-#undef PACKAGE_STRING
-#undef PACKAGE_NAME
-#undef PACKAGE
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -77,7 +70,8 @@
 #ifdef gettext_noop
 # define N_(String) gettext_noop (String)
 #else
-# define N_(String) (String)
+/* See locate.c for explanation as to why not use (String) */
+# define N_(String) String
 #endif
 
 #include "locatedb.h"
@@ -144,7 +138,7 @@ main (int argc, char **argv)
     {
       fprintf (stderr, _("Usage: %s most_common_bigrams < list > coded_list\n"),
 	       argv[0]);
-      exit (2);
+      return 2;
     }
 
   fp = fopen (argv[1], "r");
@@ -152,7 +146,7 @@ main (int argc, char **argv)
     {
       fprintf (stderr, "%s: ", argv[0]);
       perror (argv[1]);
-      exit (1);
+      return 1;
     }
 
   pathsize = oldpathsize = 1026; /* Increased as necessary by getline.  */
@@ -229,5 +223,5 @@ main (int argc, char **argv)
   free (path);
   free (oldpath);
 
-  exit (0);
+  return 0;
 }
